@@ -15,39 +15,39 @@ class GradientDescent(GeneticAlgorithm):
         self.evaluator = GradientEvaluator(self)
         self.options.declare(name='n_iterations', default=10,
                              desc='Maximum number of iterations')
-
         self.options.declare(name='x0', default=0,
                              desc='initial point')
-
         self.options.declare(name='h', default=0.1,
                              desc='step')
 
-        self.features = {'gradient': []}
-
-    def add_features(self, individuals):
-        for individual in individuals:
-            individual.features = self.features.copy()
+        self.individual_features['gradient'] = dict()
+        self.generator = RandomGenerator(self.problem.parameters, self.individual_features)
 
     def run(self):
         h = self.options["h"]
-
-        self.generator = RandomGenerator(self.problem.parameters)
         self.generator.init(self.options['max_population_size'])
-        population = self.gen_initial_population()
-        self.add_features(population.individuals)
-        self.evaluate(population.individuals)
-        self.problem.populations.append(population)
+
+        # generate
+        individuals = self.generator.generate()
+
+        # append to problem
+        for individual in individuals:
+            self.problem.individuals.append(individual)
+
+        self.evaluate(individuals)
 
         # TODO: add adaptive step size
         for it in range(self.options['max_population_number']):
-            population = Population()
-            for individual in self.problem.populations[it].individuals:
+            for individual in individuals:
                 x = []
                 for i in range(len(individual.vector)):
                     x.append(individual.vector[i] - h * individual.features['gradient'][i])
-                individual = Individual(x)
-                population.individuals.append(individual)
-            self.add_features(population.individuals)
-            self.evaluate(population.individuals)
-            self.problem.populations.append(population)
+                individual = Individual(x, self.individual_features)
+                # append to problem
+                self.problem.individuals.append(individual)
+                # sync to datastore
+                self.problem.data_store.sync_individual(individual)
+
+            self.evaluate(individuals)
+
 
