@@ -78,8 +78,10 @@ class SALibAlgorithm(Algorithm):
         for vector in self.samples_x:
             individuals.append(Individual(vector))
 
-        # set current size
-        self.population_size = len(individuals)
+        # append to problem
+        for individual in individuals:
+            self.problem.individuals.append(individual)
+
         # evaluate individuals
         self.evaluate(individuals)
 
@@ -87,9 +89,6 @@ class SALibAlgorithm(Algorithm):
             self.samples_y.append(individual.costs[0]) # TODO: fix index [0]
         self.samples_y = np.array(self.samples_y)
 
-        population = Population(individuals)
-        self.problem.populations.append(population)
-
         t = time.time() - t_s
         self.problem.logger.info("Sweep: elapsed time: {} s".format(t))
 
@@ -141,14 +140,10 @@ class Sensitivity(Algorithm):
                              desc='Maximal number of individuals in population')
 
     def run(self):
-        self.population_size = self.options['max_population_size']
-
         parameters = []
         for parameter in self.problem.parameters:
             parameters.append(float(parameter['initial_value']))
 
-        population = Population()
-
         for parameter_name in self.parameters:
             parameter_values = []
 
@@ -162,18 +157,18 @@ class Sensitivity(Algorithm):
 
             individuals = []
             for i in range(self.options['max_population_size']):
-                value = VectorAndNumbers.gen_number(selected_parameter['bounds'], selected_parameter['precision'],
-                                              'normal')
+                value = VectorAndNumbers.gen_number(selected_parameter['bounds'], selected_parameter['precision'], 'normal')
                 parameters[index] = value
                 parameter_values.append(value)
                 individual = Individual(parameters.copy())
                 individuals.append(individual)
 
-            self.evaluate(population.individuals)
-            costs = []
-            # TODO: Make also for multi-objective
-            for individual in population.individuals:
-                costs.append(individual.costs)
+            self.evaluate(individuals)
+            # costs = []
+            # # TODO: Make also for multi-objective
+            # for individual in individuals:
+            #     costs.append(individual.costs)
 
-        # append population
-        self.problem.populations.append(population)
+        # append individuals
+        for individual in individuals:
+            self.problem.individuals.append(individual)
