@@ -17,7 +17,7 @@ from abc import ABCMeta, abstractmethod
 from .utils import ConfigDictionary
 from shutil import copyfile
 
-from artap.config import config
+from .config import config
 
 
 # parse ip address
@@ -603,4 +603,4 @@ class CondorCSTJobExecutor(CondorJobExecutor):
                                request_cpus=self.request_cpus,
                                request_memory=self.request_memory,
                                hold_on_start=self.hold_on_start,
-                               desc=desc)
+                               desc=desc)
\ No newline at end of file
