@@ -6,9 +6,8 @@ from math import inf
 
 
 class Job(metaclass=ABCMeta):
-    def __init__(self, problem, population=None):
+    def __init__(self, problem):
         self.problem = problem
-        self.population = population
 
     def evaluate(self, individual):
         # try to calculate the goal function of the individual in case of failure the individual is
@@ -20,7 +19,7 @@ class Job(metaclass=ABCMeta):
 
         for i in range(5):
             # info
-            individual.info["start_time"] = time.time()
+            individual.features["start_time"] = time.time()
             t_s = time.time()
 
             # set in progress
@@ -30,7 +29,7 @@ class Job(metaclass=ABCMeta):
             constraints = self.problem.evaluate_constraints(individual)
 
             if constraints:
-                individual.feasible = sum(map(abs, constraints))
+                individual.features["feasible"] = sum(map(abs, constraints))
 
             # problem cost function evaluate only in that case when the problem fits the constraints
             try:
@@ -41,21 +40,17 @@ class Job(metaclass=ABCMeta):
 
                 # set evaluated
                 individual.state = individual.State.EVALUATED
-                # add to population
-                if self.population is not None:
-                    self.population.individuals.append(individual)
-                    individual.population_id = self.population.id
-                    individual.info["population"] = self.problem.populations.index(self.population)
                 # info
-                individual.info["finish_time"] = time.time()
-                self.problem.data_store.write(individual)
+                individual.features["finish_time"] = time.time()
+                # write to store
+                self.problem.data_store.sync_individual(individual)
                 return
 
             except (TimeoutError, RuntimeError) as e:
                 failed_individual = Individual(individual.vector)
                 failed_individual.state = individual.State.FAILED
-                if individual.feasible == 0.0:
-                    individual.feasible = inf  # TODO: genetic algorithms uses this information, i dont know the
+                if individual.features["feasible"] == 0.0:
+                    individual.features["feasible"] = inf  # TODO: genetic algorithms uses this information, i dont know the
                     # correct solution
                 self.problem.failed.append(failed_individual)
                 # in the case of failure generate new random individual
