@@ -63,7 +63,10 @@ class Evaluator(Operator):
 
     def evaluate_scalar(self, vector):
         individual = Individual(list(vector))
-        self.algorithm.problem.populations[-1].individuals.append(individual)
+
+        # add to problem
+        self.algorithm.problem.individuals.append(individual)
+
         self.job.evaluate(individual)
         return individual.costs[0]
 
@@ -100,8 +103,9 @@ class GradientEvaluator(Evaluator):
 
         self.add(individual)
         self.run()
-        self.algorithm.problem.populations[-1].individuals.append(individual)
+        self.algorithm.problem.individuals.append(individual)
         self.to_evaluate = []
+
         return individual.costs[0]
 
     def run(self):
@@ -164,7 +168,7 @@ class WorstCaseEvaluator(Evaluator):
         self.add(parent_individual)
         for individual in self.to_evaluate:
             individual.costs.append(self.job.evaluate(individual))
-        self.algorithm.problem.populations[-1].individuals.append(parent_individual)
+        self.algorithm.problem.individuals.append(parent_individual)
         self.to_evaluate = []
         return parent_individual.costs[0]
 
@@ -185,16 +189,10 @@ class WorstCaseEvaluator(Evaluator):
 
 
 class Generator(Operator):
-
-    def __init__(self, parameters=None, individual_class=Individual):
+    def __init__(self, parameters=None, features=dict()):
         super().__init__()
         self.parameters = parameters
-        self.individual_class = individual_class
-
-    def create_individual(self, vector=None):
-        if vector is None:
-            vector = []
-        return self.individual_class(vector)
+        self.features = features
 
     @abstractmethod
     def generate(self):
@@ -202,9 +200,8 @@ class Generator(Operator):
 
 
 class CustomGenerator(Generator):
-
-    def __init__(self, parameters=None, individual_class=Individual):
-        super().__init__(parameters, individual_class)
+    def __init__(self, parameters=None, features=dict()):
+        super().__init__(parameters, features)
         self.vectors = []
 
     def init(self, vectors):
@@ -213,14 +210,13 @@ class CustomGenerator(Generator):
     def generate(self):
         individuals = []
         for vector in self.vectors:
-            individuals.append(self.create_individual(vector))
+            individuals.append(Individual(vector, self.features))
         return individuals
 
 
 class RandomGenerator(Generator):
-
-    def __init__(self, parameters=None, individual_class=Individual):
-        super().__init__(parameters, individual_class)
+    def __init__(self, parameters=None, features=dict()):
+        super().__init__(parameters, features)
         self.number = 0
 
     def init(self, number):
@@ -230,7 +226,7 @@ class RandomGenerator(Generator):
         individuals = []
         for i in range(self.number):
             vector = VectorAndNumbers.gen_vector(self.parameters)
-            individuals.append(self.create_individual(vector))
+            individuals.append(Individual(vector, self.features))
         return individuals
 
 
@@ -240,8 +236,8 @@ class FullFactorGenerator(Generator):
     Number of experiments (2 ** len(parameters) - without center, 3 ** len(parameters - with center)
     """
 
-    def __init__(self, parameters=None, individual_class=Individual):
-        super().__init__(parameters, individual_class)
+    def __init__(self, parameters=None, features=dict()):
+        super().__init__(parameters, features)
         self.center = False
 
     def init(self, center):
@@ -263,7 +259,7 @@ class FullFactorGenerator(Generator):
 
         individuals = []
         for vector in df:
-            individuals.append(self.create_individual(vector))
+            individuals.append(Individual(vector, self.features))
         return individuals
 
 
@@ -273,8 +269,8 @@ class PlackettBurmanGenerator(Generator):
     Number of experiments (2 ** len(parameters) - without center, 3 ** len(parameters - with center)
     """
 
-    def __init__(self, parameters=None, individual_class=Individual):
-        super().__init__(parameters, individual_class)
+    def __init__(self, parameters=None, features=dict()):
+        super().__init__(parameters, features)
 
     def generate(self):
         dict_vars = {}
@@ -289,7 +285,7 @@ class PlackettBurmanGenerator(Generator):
 
         individuals = []
         for vector in df:
-            individuals.append(self.create_individual(vector))
+            individuals.append(Individual(vector, self.features))
         return individuals
 
 
@@ -305,8 +301,8 @@ class BoxBehnkenGenerator(Generator):
     https://en.wikipedia.org/wiki/Box%E2%80%93Behnken_design
     """
 
-    def __init__(self, parameters=None, individual_class=Individual):
-        super().__init__(parameters, individual_class, )
+    def __init__(self, parameters=None, features=dict()):
+        super().__init__(parameters, features)
 
     def generate(self):
         dict_vars = {}
@@ -321,7 +317,7 @@ class BoxBehnkenGenerator(Generator):
 
         individuals = []
         for vector in df:
-            individuals.append(self.create_individual(vector))
+            individuals.append(Individual(vector, self.features))
         return individuals
 
 
@@ -330,8 +326,8 @@ class LHSGenerator(Generator):
     Builds a Latin Hypercube design dataframe from a dictionary of factor/level ranges.
     """
 
-    def __init__(self, parameters=None, individual_class=Individual):
-        super().__init__(parameters, individual_class)
+    def __init__(self, parameters=None, features=dict()):
+        super().__init__(parameters, features)
         self.number = 0
 
     def init(self, number):
@@ -350,12 +346,11 @@ class LHSGenerator(Generator):
 
         individuals = []
         for vector in df:
-            individuals.append(self.create_individual(vector))
+            individuals.append(Individual(vector, self.features))
         return individuals
 
 
 class Mutator(Operator):
-
     def __init__(self, parameters, probability):
         super().__init__()
         self.parameters = parameters
@@ -387,7 +382,7 @@ class SimpleMutator(Mutator):
             else:
                 vector.append(p.vector[i])
 
-        p_new = p.__class__(vector)
+        p_new = Individual(vector, p.features)
         return p_new
 
 
@@ -414,7 +409,7 @@ class PmMutator(Mutator):
             else:
                 vector.append(parent.vector[i])
 
-        return parent.__class__(vector)
+        return Individual(vector, parent.features)
 
     def pm_mutation(self, x, lb, ub):
         """
@@ -473,7 +468,7 @@ class UniformMutator(Mutator):
             else:
                 vector.append(parent.vector[i])
 
-        return parent.__class__(vector)
+        return Individual(vector, parent.features)
 
     def uniform_mutation(self, x, lb, ub):
 
@@ -505,7 +500,7 @@ class NonUniformMutation(Mutator):
             else:
                 vector.append(parent.vector[i])
 
-        return parent.__class__(vector)
+        return Individual(vector, parent.features)
 
     def non_uniform_mutation(self, x, lb, ub, current_iteration):
 
@@ -943,6 +938,7 @@ class ParetoDominance(Dominance):
         """
         Here, p and q are tuples, which contains the (feasibility index, cost vector)
         """
+        # assert len(p) == len(q)
 
         # first check constraint violation, the last item is the feasibility, which is a real number if its zero,
         # it means that the solution is feasible
@@ -996,7 +992,7 @@ class Selector(Operator):
     def select(self, population):
         pass
 
-    def pop_acceptance(self, population, individual):
+    def pop_acceptance(self, individuals, individual):
         """
         This function differs from add() function that it preserves the length of the _content in the archive class.
         :return:
@@ -1005,8 +1001,8 @@ class Selector(Operator):
         dominates = []
         dominated = False
 
-        for i in range(len(population)):
-            flag = self.dominance.compare(individual.costs_signed, population[i].costs_signed)
+        for i in range(len(individuals)):
+            flag = self.dominance.compare(individual.costs_signed, individuals[i].costs_signed)
 
             if flag == 1:
                 dominates.append(i)
@@ -1014,12 +1010,12 @@ class Selector(Operator):
                 dominated = True
 
         if len(dominates) > 0:
-            del population[random.choice(dominates)]
-            population.append(individual)
+            del individuals[random.choice(dominates)]
+            individuals.append(individual)
 
         elif not dominated:
-            population.remove(random.choice(population))
-            population.append(individual)
+            individuals.remove(random.choice(individuals))
+            individuals.append(individual)
 
         return
 
@@ -1151,7 +1147,14 @@ def nondominated_cmp(p, q):
 
 
 class DummySelector(Selector):
+    def __init__(self, parameters):
+        super().__init__(parameters)
 
+    def select(self, individuals):
+        return individuals
+
+
+class CopySelector(Selector):
     def __init__(self, parameters):
         super().__init__(parameters)
 
@@ -1159,19 +1162,19 @@ class DummySelector(Selector):
         selection = []
         for individual in individuals:
             candidate = Individual(individual.vector)
-            candidate.costs = individual.costs.copy()
-            candidate.features = individual.features.copy()
+            candidate.costs = deepcopy(individual.costs)
+            candidate.features = deepcopy(individual.features)
+            candidate.population_id = -1
             selection.append(candidate)
         return selection
 
 
 class TournamentSelector(Selector):
-
     def __init__(self, parameters, dominance=ParetoDominance, epsilons=None):
         super().__init__(parameters)
         self.dominance = dominance(epsilons=epsilons)
 
-    def select(self, population):
+    def select(self, individuals):
         """
         Binary tournament selection:
 
@@ -1179,12 +1182,11 @@ class TournamentSelector(Selector):
         back the dominant (or random).
         """
 
-        if len(population) == 1:
-            selected = population[0]
-
+        if len(individuals) == 1:
+            selected = individuals[0]
         else:
             # Sampling two individuals without a replacement
-            candidates = random.sample(population, 2)
+            candidates = random.sample(individuals, 2)
 
             # they should have this at this stage
             if candidates[0].features['front_number'] < candidates[1].features['front_number']:
@@ -1203,6 +1205,7 @@ class TournamentSelector(Selector):
 
         return selected
 
+
 class FireflyStep(Operator):
     """
       This operator makes the following
@@ -1268,7 +1271,6 @@ class FireflyStep(Operator):
         return
 
 
-
 class Crossover(Operator):
 
     def __init__(self, parameters, probability):
@@ -1282,7 +1284,6 @@ class Crossover(Operator):
 
 
 class SimpleCrossover(Crossover):
-
     def __init__(self, parameters, probability):
         super().__init__(parameters, probability)
 
@@ -1307,8 +1308,8 @@ class SimpleCrossover(Crossover):
             parent_a = parameter1
             parent_b = parameter2
 
-        offspring_a = p1.__class__(parent_a)
-        offspring_b = p2.__class__(parent_b)
+        offspring_a = Individual(parent_a, p1.features)
+        offspring_b = Individual(parent_b, p2.features)
 
         return offspring_a, offspring_b
 
@@ -1381,4 +1382,4 @@ class SimulatedBinaryCrossover(Crossover):
                         else:
                             x1[i], x2[i] = c1, c2
 
-        return p1.__class__(x1), p2.__class__(x2)
+        return Individual(x1, p1.features), Individual(x2, p2.features)
