@@ -2,22 +2,22 @@ from .individual import Individual
 
 
 class Population:
-
     counter = 0
 
     def __init__(self, individuals=None):
         self.id = Population.counter
         Population.counter += 1
-        self.individuals = []
-        if individuals is not None:
-            self.set_individuals(individuals)
-        self.pareto_vectors: list = []
-        self.pareto_costs: list = []
 
-    def set_individuals(self, individuals=None):
-        self.individuals = individuals
-        for individual in self.individuals:
-            individual.population_id = self.id
+        if individuals is not None:
+            self.individuals = individuals
+            for individual in individuals:
+                individual.population_id = self.id
+        else:
+            assert 1
+            self.individuals = []
+
+        # self.pareto_vectors: list = []
+        # self.pareto_costs: list = []
 
     def __str__(self):
         string = "individuals: {}\n".format(len(self.individuals))
@@ -29,4 +29,5 @@ class Population:
         if not isinstance(other, Population):
             return NotImplemented
 
-        return self.individuals == other.individuals and self.pareto_vectors == other.pareto_vectors and self.pareto_costs == other.pareto_costs
\ No newline at end of file
+        return self.individuals == other.individuals
+        # and self.pareto_vectors == other.pareto_vectors and self.pareto_costs == other.pareto_costs
\ No newline at end of file
