@@ -2,9 +2,10 @@
  Module is dedicated to describe optimization problem.
 """
 
-from .datastore import FileDataStore, DummyDataStore
+from .datastore import TinyDataStore, SqliteDataStore, DummyDataStore
 from .utils import ConfigDictionary
 from .surrogate import SurrogateModelEval
+from .population import Population
 from abc import abstractmethod
 
 import logging
@@ -13,6 +14,7 @@ import tempfile
 import os
 import shutil
 import atexit
+import platform
 
 CRITICAL = logging.CRITICAL
 FATAL = logging.FATAL
@@ -89,14 +91,18 @@ class Problem:
         self.costs: dict = dict()
         self.signs = []
 
+        self.info = {"processor": platform.processor(),
+                     "system": platform.system(),
+                     "python": platform.python_version(),
+                     "hostname": platform.node()}
+
+
         # populations
-        self.populations = []
-        self.archive = None
-        self.buffer = [] # storage for potential individuals, can be used if calculation fails
-        self.failed = [] # storage for failed individuals
+        self.individuals = []
         self.data_store = None
         self.monitor_service = None
         self.executor = None
+        self.failed = []  # storage for failed individuals
         self.data_store = DummyDataStore()
         self.output_files = None
 
@@ -120,11 +126,41 @@ class Problem:
         # clean up
         atexit.register(self.cleanup)
 
+    def populations(self):
+        individuals = {}
+        for individual in self.individuals:
+            if individual.population_id not in individuals:
+                individuals[individual.population_id] = []
+            individuals[individual.population_id].append(individual)
+
+        return individuals
+
+    def population(self, id):
+        individuals = []
+        for individual in self.individuals:
+            if individual.population_id == id:
+                individuals.append(individual)
+
+        return individuals
+
+    def last_population(self):
+        # TODO: temporary solution - faster implementation with TinyDB
+
+        # find max index
+        max_index = -1
+        for individual in self.individuals:
+            if individual.population_id > max_index:
+                max_index = individual.population_id
+
+        # add to population
+        return self.population(max_index)
+
     def to_dict(self):
         output = {'name': self.name, 'description': self.description, 'parameters': self.parameters}
         return output
 
-    def from_dict(self):
+    @staticmethod
+    def from_dict():
         pass
 
     def cleanup(self):
@@ -206,9 +242,9 @@ class Problem:
 
 
 class ProblemViewDataStore(Problem):
-    def __init__(self, database_name):
+    def __init__(self, database_name, class_datastore=SqliteDataStore):
         super().__init__()
-        self.data_store = FileDataStore(self, database_name=database_name, mode="read")
+        self.data_store = class_datastore(self, database_name=database_name, mode="read")
 
     def set(self, **kwargs):
         pass
